@@ -623,21 +623,30 @@ def make_asset_zip(asset_dir_path, destination_directory=None):
                 for f in files:
                     skip = False
                     file_path = os.path.join(root, f)
+                    log.debug('Evaluating file: {f}'.format(f=file_path))
 
-                    # Skip files in the ignore directories list
+                    # Skip files in the ignore_dirs list
                     for ignore_dir in ignore_dirs:
                         if ignore_dir in file_path:
-                            skip = True
-                            break
+                            matching_item = False
+                            for file_path_component in file_path.split(os.sep):
+                                if file_path_component == ignore_dir:
+                                    matching_item = True
+                            if matching_item:
+                                test_dir = file_path[:file_path.index(ignore_dir) + len(ignore_dir)]
+                                if os.path.isdir(test_dir):
+                                    log.info('File is in an ignore directory {d}: {f}'.format(d=ignore_dir, f=file_path))
+                                    skip = True
 
-                    # Skip file in the ignore files list
+                    # Skip file in the ignore_files list
                     for ignore_file in ignore_files:
                         if f.startswith(ignore_file):
+                            log.info('File starts with ignored file prefix {p}: {f}'.format(p=ignore_file, f=file_path))
                             skip = True
-                            break
 
-                    # Skip if the file ends with the specified extension
+                    # Skip if the file ends with one of the items in ignore_file_extensions
                     if ignore_by_extension(item_path=file_path):
+                        log.info('File has an ignored extension: {f}'.format(f=file_path))
                         skip = True
 
                     if skip:
@@ -649,7 +658,7 @@ def make_asset_zip(asset_dir_path, destination_directory=None):
                     if archive_name.startswith('/'):
                         log.debug('Trimming the leading char: [/]')
                         archive_name = archive_name[1:]
-                    log.info('Adding to archive as: {a}'.format(a=archive_name))
+                    log.info('Adding file to archive as: {a}'.format(a=archive_name))
                     zip_w.write(file_path, archive_name)
     except Exception as exc:
         raise AssetZipCreationError('Unable to create zip file: {f}'.format(f=asset_info.asset_zip_path)) from exc
@@ -893,7 +902,8 @@ def import_update(asset_dir, dest_dir, visibility=None, import_only=False, updat
         # If there is not an existing asset ID, import and append the resulting ID
         else:
             if not update_only:
-                print('No existing asset ID found, attempting to import asset zip: {f}'.format(f=asset_info.asset_zip_path))
+                print('No existing asset ID found, attempting to import asset zip: {f}'.format(
+                    f=asset_info.asset_zip_path))
                 asset_info, asset_id, result = import_asset(cons3rt_api=c5t, asset_info=asset_info)
                 if result:
                     print('Imported asset successfully')
@@ -1126,6 +1136,7 @@ def main():
     parser.add_argument('--asset_type', help='Set to: containers, software')
     parser.add_argument('--category_ids', help='List of category IDs to filter on')
     parser.add_argument('--community', help='Include to retrieve community assets', action='store_true')
+    parser.add_argument('--config', help='Path to a config file to load', required=False)
     parser.add_argument('--dest_dir', help='Destination directory for the asset zip (default is Downloads)')
     parser.add_argument('--expanded', help='Include to retrieve expanded info on assets', action='store_true')
     parser.add_argument('--id', help='Asset ID to download or update')
@@ -1165,6 +1176,12 @@ def main():
     else:
         asset_dir = working_dir
 
+    # Determine if a config file was provided
+    config_file = None
+    if args.config:
+        config_file = args.config
+        print('Using cons3rt API config file: {f}'.format(f=config_file))
+
     # Determine the destination directory
     if args.dest_dir:
         dest_dir_provided = args.dest_dir.strip()
@@ -1218,18 +1235,19 @@ def main():
         res = download_cli(args)
     elif command == 'import':
         asset, res, err = import_update(asset_dir=asset_dir, dest_dir=dest_dir, import_only=True,
-                                        visibility=visibility, log_level='WARNING')
+                                        visibility=visibility, log_level='WARNING', config_file=config_file)
     elif command == 'query':
         res = query_assets_args(args)
     elif command == 'queryids':
         res = query_assets_args(args, id_only=True)
     elif command == 'update':
         asset, res, err = import_update(asset_dir=asset_dir, dest_dir=dest_dir, visibility=visibility,
-                                        log_level='WARNING', keep_asset_zip=keep, update_asset_id=asset_id)
+                                        log_level='WARNING', keep_asset_zip=keep, update_asset_id=asset_id,
+                                        config_file=config_file)
     elif command == 'updateonly':
         asset, res, err = import_update(asset_dir=asset_dir, dest_dir=dest_dir, visibility=visibility,
                                         log_level='WARNING', keep_asset_zip=keep, update_only=True,
-                                        update_asset_id=asset_id)
+                                        update_asset_id=asset_id, config_file=config_file)
     elif command == 'validate':
         validate(asset_dir=asset_dir)
     return res
