@@ -14,7 +14,6 @@ import fileinput
 import re
 import sys
 import zipfile
-import socket
 import contextlib
 import signal
 import time
@@ -135,11 +134,12 @@ def run_command_large_buffer(command, timeout_sec=3600.0):
     :return: Dict containing stdout, stderr, and return code
     :raises CommandError
     """
-    log = logging.getLogger(mod_logger + '.run_command')
+    log = logging.getLogger(mod_logger + '.run_command_large_buffer')
     if not isinstance(command, list):
         raise CommandError('command arg must be a list')
     command_str = ' '.join(command)
-    log.debug('Running command: {c}'.format(c=command_str))
+    log.error('Running command: {c}'.format(c=command_str))
+    print('Running command: {c}'.format(c=command_str))
     subproc = subprocess.Popen(command, stdin=subprocess.DEVNULL, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
     pid = subproc.pid
     log.debug('Opened up PID {p} with a timeout of {s} sec...'.format(p=pid, s=str(timeout_sec)))
@@ -415,7 +415,7 @@ def source(script):
     if not os.path.isfile(script):
         raise CommandError('File not found: {f}'.format(f=script))
     log.debug('Attempting to source script: {f}'.format(f=script))
-    command = ['bash', '-c', '. {s}; env;'.format(s=script)]
+    command = ['bash', '-c', '. {s} >> /dev/null 2>&1; env;'.format(s=script)]
     try:
         result = run_command(command, timeout_sec=10.0, output=True, print_output=False)
     except CommandError as exc:
@@ -428,7 +428,7 @@ def source(script):
     for env_var in env_vars:
         entry = env_var.split("=", 1)
         if len(entry) != 2:
-            log.warning('Not in prop=value format, skipping: {p}'.format(p=env_var))
+            log.debug('Line not in prop=value format, skipping: {p}'.format(p=env_var))
             continue
         try:
             env[entry[0]] = entry[1]
@@ -1172,12 +1172,13 @@ def set_remote_host_environment_variable(host, variable_name, variable_value, en
         log.info('Environment variable {v} set to {n} on host {h}'.format(v=variable_name, n=variable_value, h=host))
 
 
-def run_remote_command(host, command, timeout_sec=5.0, port=22, output=True):
+def run_remote_command(host, command, user=None, timeout_sec=5.0, port=22, output=True):
     """Retrieves the value of an environment variable of a
     remote host over SSH
 
     :param host: (str) host to query
     :param command: (str) command
+    :param user: (str) username on the remote host
     :param timeout_sec (float) seconds to wait before killing the command
     :param port (int) SSH port to connect on the remote machine
     :param output (bool) Set False to ignore command output
@@ -1191,11 +1192,19 @@ def run_remote_command(host, command, timeout_sec=5.0, port=22, output=True):
     if not isinstance(command, str):
         msg = 'command argument must be a string'
         raise TypeError(msg)
+    if user:
+        if not isinstance(user, str):
+            msg = 'user argument must be a string'
+            raise TypeError(msg)
     log.debug('Running remote command on host: {h}: {c}...'.format(h=host, c=command))
     remote_command = ['ssh']
     if port != 22:
         remote_command += ['-p', str(port)]
-    remote_command += ['{h}'.format(h=host), '{c}'.format(c=command)]
+    if user:
+        remote_command += ['{u}@{h}'.format(u=user, h=host)]
+    else:
+        remote_command += ['{h}'.format(h=host)]
+    remote_command += ['{c}'.format(c=command)]
     try:
         result = run_command(remote_command, timeout_sec=timeout_sec, output=output)
         code = result['code']
