@@ -98,7 +98,7 @@ def main():
     parser = argparse.ArgumentParser(description='CONS3RT command line interface (CLI)')
     parser.add_argument('command', help='Command for the cons3rt CLI')
     parser.add_argument('subcommands', help='Optional command subtype', nargs='*')
-    parser.add_argument('--active', help='Process only active runs', required=False, action='store_true')
+    parser.add_argument('--active', help='Process only active items', required=False, action='store_true')
     parser.add_argument('--all', help='All action relative to the command provided', action='store_true')
     parser.add_argument('--assets', help='Specify to run the asset reports', action='store_true')
     parser.add_argument('--clean_all_runs', help='Clean all runs from a cloudspace', action='store_true')
@@ -108,6 +108,7 @@ def main():
     parser.add_argument('--config', help='Path to a config file to load', required=False)
     parser.add_argument('--delete', help='Delete action relative to the command provided', action='store_true')
     parser.add_argument('--delete_inactive_runs', help='Delete inactive runs from a cloudspace', action='store_true')
+    parser.add_argument('--expired', help='Process only expired items', required=False, action='store_true')
     parser.add_argument('--id', help='ID relative to the command provided', required=False)
     parser.add_argument('--ids', help='List of IDs relative to the command provided', required=False)
     parser.add_argument('--impact_level', help='Impact level of the resources', required=False)
@@ -129,6 +130,7 @@ def main():
     parser.add_argument('--runs', help='Run ID(s) relative to the command provided', required=False)
     parser.add_argument('--share', help='Share templates', action='store_true')
     parser.add_argument('--state', help='Project membership state')
+    parser.add_argument('--unexpired', help='Process only unexpired items', required=False, action='store_true')
     parser.add_argument('--unlock', help='Remove run locks before taking action', action='store_true')
     parser.add_argument('--username', help='username relative to the CLI call', required=False)
     parser.add_argument('--yaml', help='Path to YAML input/output file for to the CLI call', required=False)
