@@ -262,17 +262,27 @@ class Cons3rtCli(object):
 
     @staticmethod
     def print_teams(teams_list):
-        msg = 'ID\tName\n'
+        msg = 'ID\tName\t\t\tState\t\tExpiration\n'
         for team in teams_list:
-            if 'id' in team:
+            if 'id' in team.keys():
                 msg += str(team['id'])
             else:
                 msg += '      '
             msg += '\t'
-            if 'name' in team:
+            if 'name' in team.keys():
                 msg += team['name']
             else:
                 msg += '                '
+            msg += '\t\t'
+            if 'state' in team.keys():
+                msg += team['state']
+            else:
+                msg += '        '
+            msg += '\t\t'
+            if 'expirationDate' in team.keys():
+                msg += team['expirationDate']
+            else:
+                msg += '                '
             msg += '\n'
         print(msg)
 
@@ -464,6 +474,21 @@ class CloudCli(Cons3rtCli):
             raise Cons3rtCliError(msg) from exc
 
     def delete_clouds(self):
+        if len(self.subcommands) == 1:
+            self.delete_clouds_only()
+        elif len(self.subcommands) > 1:
+            delete_subcommand = self.subcommands[1]
+            valid_delete_subcommands = [
+                'cloudspaces'
+            ]
+            if delete_subcommand not in valid_delete_subcommands:
+                self.err('Unrecognized cloud delete subcommand [{c}], options: {o}'.format(
+                    c=delete_subcommand, o=','.join(valid_delete_subcommands)))
+                return False
+            if delete_subcommand == 'cloudspaces':
+                self.delete_cloudspaces()
+
+    def delete_clouds_only(self):
         if not self.ids:
             msg = '--id or --ids arg required to specify the cloud IDs to delete'
             self.err(msg)
@@ -476,6 +501,27 @@ class CloudCli(Cons3rtCli):
                 self.err(msg)
                 raise Cons3rtCliError(msg) from exc
 
+    def delete_cloudspaces(self):
+        if not self.ids:
+            msg = '--id or --ids arg required to specify the cloud IDs to delete'
+            self.err(msg)
+            raise Cons3rtCliError(msg)
+        for cloud_id in self.ids:
+            try:
+                deleted_cloudspaces, undeleted_cloudspaces = self.c5t.delete_virtualization_realms_for_cloud(
+                    cloud_id=cloud_id)
+            except Cons3rtApiError as exc:
+                msg = 'Problem deleting VRs in cloud ID cloud ID: {c}\n{e}'.format(c=str(cloud_id), e=str(exc))
+                self.err(msg)
+                raise Cons3rtCliError(msg) from exc
+            print('Deallocated/unregistered {n} cloudspaces for cloud {c}:'.format(
+                n=str(len(deleted_cloudspaces)), c=str(cloud_id)))
+            self.print_cloudspaces(cloudspaces_list=deleted_cloudspaces)
+            if len(undeleted_cloudspaces) > 0:
+                print('Unable to deallocate/unregister {n} cloudspaces for cloud {c}:'.format(
+                    n=str(len(undeleted_cloudspaces)), c=str(cloud_id)))
+                self.print_cloudspaces(cloudspaces_list=undeleted_cloudspaces)
+
     def retrieve_clouds(self):
         if not self.ids:
             msg = '--id or --ids arg required to retrieve cloud details'
@@ -563,9 +609,11 @@ class CloudspaceCli(Cons3rtCli):
         self.valid_subcommands = [
             'allocate',
             'deallocate',
+            'delete_inactive_runs',
             'list',
             'template',
             'register',
+            'release_active_runs',
             'retrieve',
             'unregister'
         ]
@@ -588,6 +636,11 @@ class CloudspaceCli(Cons3rtCli):
                 self.deallocate_cloudspace()
             except Cons3rtCliError:
                 return False
+        elif self.subcommands[0] == 'delete_inactive_runs':
+            try:
+                self.delete_inactive_runs()
+            except Cons3rtCliError:
+                return False
         elif self.subcommands[0] == 'list':
             try:
                 self.list_cloudspace()
@@ -598,6 +651,11 @@ class CloudspaceCli(Cons3rtCli):
                 self.register_cloudspace()
             except Cons3rtCliError:
                 return False
+        elif self.subcommands[0] == 'release_active_runs':
+            try:
+                self.release_active_runs()
+            except Cons3rtCliError:
+                return False
         elif self.subcommands[0] == 'retrieve':
             try:
                 self.retrieve_cloudspace()
@@ -673,12 +731,16 @@ class CloudspaceCli(Cons3rtCli):
 
     def delete_inactive_runs_from_cloudspace(self, cloudspace_id):
         try:
-            self.c5t.delete_inactive_runs_in_virtualization_realm(vr_id=cloudspace_id)
+            deleted_runs, not_deleted_runs  = self.c5t.delete_inactive_runs_in_virtualization_realm(vr_id=cloudspace_id)
         except Cons3rtApiError as exc:
             msg = 'There was a problem deleting inactive runs from cloudspace ID: {i}\n{e}'.format(
                 i=str(cloudspace_id), e=str(exc))
             self.err(msg)
             raise Cons3rtCliError(msg) from exc
+        print('Deleted {n} inactive runs from cloudspace: {i}'.format(n=str(len(deleted_runs)), i=str(cloudspace_id)))
+        if len(not_deleted_runs) > 0:
+            print('Unable to delete {n} inactive runs from cloudspace: {i}'.format(
+                n=str(len(not_deleted_runs)), i=str(cloudspace_id)))
 
     def delete_templates(self):
         for cloudspace_id in self.ids:
@@ -844,12 +906,16 @@ class CloudspaceCli(Cons3rtCli):
 
     def release_active_runs_from_cloudspace(self, cloudspace_id):
         try:
-            self.c5t.release_active_runs_in_virtualization_realm(vr_id=cloudspace_id)
+            released_runs, not_released_runs = self.c5t.release_active_runs_in_virtualization_realm(vr_id=cloudspace_id)
         except Cons3rtApiError as exc:
             msg = 'There was a problem releasing active runs from cloudspace ID: {i}\n{e}'.format(
                 i=str(cloudspace_id), e=str(exc))
             self.err(msg)
             raise Cons3rtCliError(msg) from exc
+        print('Release {n} active runs from cloudspace: {i}'.format(n=str(len(released_runs)), i=str(cloudspace_id)))
+        if len(not_released_runs) > 0:
+            print('Unable to release {n} active runs from cloudspace: {i}'.format(
+                n=str(len(not_released_runs)), i=str(cloudspace_id)))
 
     def retrieve_cloudspace(self):
         if not self.ids:
@@ -1079,6 +1145,7 @@ class ProjectCli(Cons3rtCli):
     def __init__(self, args, subcommands):
         Cons3rtCli.__init__(self, args=args, subcommands=subcommands)
         self.valid_subcommands = [
+            'get',
             'list',
             'members',
             'run'
@@ -1092,6 +1159,11 @@ class ProjectCli(Cons3rtCli):
         if self.subcommands[0] not in self.valid_subcommands:
             self.err('Unrecognized command: {c}'.format(c=self.subcommands[0]))
             return False
+        if self.subcommands[0] == 'get':
+            try:
+                self.get_project()
+            except Cons3rtCliError:
+                return False
         if self.subcommands[0] == 'list':
             try:
                 self.list_projects()
@@ -1109,6 +1181,21 @@ class ProjectCli(Cons3rtCli):
                 return False
         return True
 
+    def get_project(self):
+        if not self.ids:
+            msg = '--id or --ids arg required to specify the project ID(s)'
+            self.err(msg)
+            raise Cons3rtCliError(msg)
+        for project_id in self.ids:
+            try:
+                project_details = self.c5t.get_project_details(project_id=project_id)
+            except Cons3rtApiError as exc:
+                msg = 'There was a problem getting details for project: {i}\n{e}'.format(i=str(project_id), e=str(exc))
+                self.err(msg)
+                raise Cons3rtCliError(msg) from exc
+            # TODO Do something better here
+            print(str(project_details))
+
     def members(self):
         if not self.ids:
             msg = '--id or --ids arg required to specify the project ID(s)'
@@ -1546,10 +1633,13 @@ class TeamCli(Cons3rtCli):
 
     def generate_report(self, team_id):
         load = False
+        config = None
         if self.args.load:
             load = True
+        if self.args.config:
+            config = self.args.config
         try:
-            generate_team_report(team_id=team_id, load=load)
+            generate_team_report(team_id=team_id, load=load, cons3rt_api=self.c5t)
         except Cons3rtReportsError as exc:
             msg = 'Problem generating report for team ID: {i}\n{e}'.format(i=str(team_id), e=str(exc))
             self.err(msg)
@@ -1557,7 +1647,7 @@ class TeamCli(Cons3rtCli):
 
     def generate_asset_report(self, team_id):
         try:
-            generate_team_asset_report(team_id=team_id)
+            generate_team_asset_report(team_id=team_id, cons3rt_api=self.c5t)
         except Cons3rtReportsError as exc:
             msg = 'Problem generating asset report for team ID: {i}\n{e}'.format(i=str(team_id), e=str(exc))
             self.err(msg)
@@ -1565,9 +1655,15 @@ class TeamCli(Cons3rtCli):
 
     def list_team_managers(self):
         results = []
+        not_expired = False
+        active_only = False
+        if self.args.unexpired:
+            not_expired = True
+        if self.args.active:
+            active_only = True
         if not self.ids:
             # No ids specified, getting a list for all teams
-            results += self.c5t.list_team_managers()
+            results += self.c5t.list_team_managers(not_expired=not_expired, active_only=active_only)
         else:
             # Generate a list for each team ID specified
             for team_id in self.ids:
@@ -1576,8 +1672,14 @@ class TeamCli(Cons3rtCli):
 
     def list_teams(self):
         teams = []
+        not_expired = False
+        active_only = False
+        if self.args.unexpired:
+            not_expired = True
+        if self.args.active:
+            active_only = True
         try:
-            teams += self.c5t.list_teams()
+            teams += self.c5t.list_teams(not_expired=not_expired, active_only=active_only)
         except Cons3rtApiError as exc:
             msg = 'There was a problem listing teams\n{e}'.format(e=str(exc))
             self.err(msg)
