@@ -195,6 +195,60 @@ class Cons3rtClient:
             msg = 'The HTTP response contains a bad status code'
             raise Cons3rtClientError(msg) from exc
 
+    def assign_role_to_project_member(self, project_id, username, role):
+        """Assigns the provided role to the username in the project ID
+        :param project_id: (int) project ID
+        :param username: (str) CONS3RT username
+        :param role: (str) project role
+        :return: (bool) True if successful
+        :raises: Cons3rtClientError
+        """
+        # Set the target URL
+        target = 'projects/{i}/members/roles/?username={u}&role={r}'.format(i=str(project_id), u=username, r=role)
+
+        # Add the user to the project
+        try:
+            response = self.http_client.http_put(rest_user=self.user, target=target)
+        except Cons3rtClientError as exc:
+            msg = 'Unable to assign role [{r}] to user [{u}] in project ID: {i}'.format(
+                r=role, u=username, i=str(project_id))
+            raise Cons3rtClientError(msg) from exc
+
+        # Check the response
+        try:
+            result = self.http_client.parse_response(response=response)
+        except Cons3rtClientError as exc:
+            msg = 'The HTTP response contains a bad status code'
+            raise Cons3rtClientError(msg) from exc
+        return result
+
+    def unassign_role_from_project_member(self, project_id, username, role):
+        """Assigns the provided role to the username in the project ID
+        :param project_id: (int) project ID
+        :param username: (str) CONS3RT username
+        :param role: (str) project role
+        :return: (bool) True if successful
+        :raises: Cons3rtClientError
+        """
+        # Set the target URL
+        target = 'projects/{i}/members/roles/?username={u}&role={r}'.format(i=str(project_id), u=username, r=role)
+
+        # Add the user to the project
+        try:
+            response = self.http_client.http_delete(rest_user=self.user, target=target)
+        except Cons3rtClientError as exc:
+            msg = 'Unable to unassign role [{r}] from user [{u}] in project ID: {i}'.format(
+                r=role, u=username, i=str(project_id))
+            raise Cons3rtClientError(msg) from exc
+
+        # Check the response
+        try:
+            result = self.http_client.parse_response(response=response)
+        except Cons3rtClientError as exc:
+            msg = 'The HTTP response contains a bad status code'
+            raise Cons3rtClientError(msg) from exc
+        return result
+
     def create_system(self, system_data):
         """Creates a system and returns the system ID
 
@@ -1058,7 +1112,7 @@ class Cons3rtClient:
         return drs
 
     def list_all_deployment_runs_in_virtualization_realm(self, vr_id, search_type='SEARCH_ALL'):
-        """Lists all of the deployment runs in a virtualization realm by page
+        """Lists the deployment runs in a virtualization realm by page
 
         :param vr_id: (int) ID of the virtualization realm
         :param search_type: (str) search type
@@ -1069,8 +1123,6 @@ class Cons3rtClient:
         page_num = 0
         max_results = 40
         while True:
-            print('Retrieving deployment runs in virtualization realm [{i}]: page {p}'.format(
-                i=str(vr_id), p=str(page_num)))
             try:
                 page_of_drs = self.list_deployment_runs_in_virtualization_realm(
                     vr_id=vr_id,
@@ -1828,11 +1880,34 @@ class Cons3rtClient:
         print('Retrieved a total of {n} software assets'.format(n=str(len(software_assets))))
         return software_assets
 
+    def retrieve_test_asset(self, asset_id):
+        """Retrieves details for the test asset
+
+        :param asset_id: (int) asset ID
+        :return: (dict) details about the test asset
+        :raises: Cons3rtClientError
+        """
+        target = 'testassets/{i}'.format(i=str(asset_id))
+        try:
+            response = self.http_client.http_get(
+                rest_user=self.user,
+                target=target
+            )
+        except Cons3rtClientError as exc:
+            raise Cons3rtClientError('Problem retrieving test asset: {i}'.format(i=str(asset_id))) from exc
+        try:
+            result = self.http_client.parse_response(response=response)
+        except Cons3rtClientError as exc:
+            msg = 'The HTTP response contains a bad status code'
+            raise Cons3rtClientError(msg) from exc
+        test_asset = json.loads(result)
+        return test_asset
+
     def retrieve_test_assets(self, asset_type=None, community=False, category_ids=None, expanded=False,
                              max_results=40, page_num=0):
         """Get a list of test assets
 
-        :param asset_type: (str) the test asset type, defaults to null
+        :param asset_type: (str) the test asset type, defaults to null: "UNKNOWN" "NESSUS" "SCRIPT" "POWERSHELL" "MOCK"
         :param community: (bool) the boolean to include community assets
         :param category_ids: (list) the list of categories to filter by
         :param expanded: (bool) whether to retrieve expanded info
