@@ -12,6 +12,7 @@ from botocore.client import ClientError
 from .awsutil import get_boto3_client
 from .exceptions import IamUtilError
 from .logify import Logify
+from .network import validate_ip_address
 
 
 __author__ = 'Joe Yennaco'
@@ -38,12 +39,25 @@ class IamUtil(object):
     def add_policy_statement(self, policy_arn, statement):
         return add_policy_statement(client=self.client, policy_arn=policy_arn, statement=statement)
 
+    def add_source_ip_addresses_to_cons3rt_bucket_policy(self, policy_arn, source_ip_addresses):
+        return add_source_ip_addresses_to_cons3rt_bucket_policy(
+            client=self.client, policy_arn=policy_arn, source_ip_addresses=source_ip_addresses)
+
+    def add_user_to_group(self, user_name, group_name):
+        return add_user_to_group(client=self.client, user_name=user_name, group_name=group_name)
+
     def attach_policy_to_group(self, group_name, policy_arn):
         return attach_policy_to_group(client=self.client, group_name=group_name, policy_arn=policy_arn)
 
     def attach_policy_to_role(self, role_name, policy_arn):
         return attach_policy_to_role(client=self.client, role_name=role_name, policy_arn=policy_arn)
 
+    def create_access_key_for_user(self, user_name):
+        return create_access_key_for_user(client=self.client, user_name=user_name)
+
+    def create_first_access_key_for_user(self, user_name):
+        return create_first_access_key_for_user(client=self.client, user_name=user_name)
+
     def create_default_policy_version(self, policy_arn, json_policy_data):
         return create_default_policy_version(client=self.client, policy_arn=policy_arn,
                                              json_policy_data=json_policy_data)
@@ -67,9 +81,21 @@ class IamUtil(object):
         return create_role(client=self.client, role_name=role_name, role_policy=role_policy, path=path,
                            description=description, max_session_duration_sec=max_session_duration_sec)
 
+    def create_user(self, user_name, path='/'):
+        return create_user(client=self.client, user_name=user_name, path=path)
+
+    def delete_access_key(self, user_name, access_key_id):
+        return delete_access_key(client=self.client, user_name=user_name, access_key_id=access_key_id)
+
+    def delete_all_access_keys_for_user(self, user_name):
+        return delete_all_access_keys_for_user(client=self.client, user_name=user_name)
+
     def delete_oldest_policy_version(self, policy_arn):
         return delete_oldest_policy_version(client=self.client, policy_arn=policy_arn)
 
+    def get_cons3rt_bucket_policy_source_ip_addresses(self, policy_arn):
+        return get_cons3rt_bucket_policy_source_ip_addresses(client=self.client, policy_arn=policy_arn)
+
     def get_default_policy_document(self, policy_arn):
         return get_default_policy_document(client=self.client, policy_arn=policy_arn)
 
@@ -79,6 +105,9 @@ class IamUtil(object):
     def get_policy_version(self, policy_arn, version_id):
         return get_policy_version(client=self.client, policy_arn=policy_arn, version_id=version_id)
 
+    def list_access_keys_for_user(self, user_name):
+        return list_access_keys_for_user(client=self.client, user_name=user_name)
+
     def list_groups(self, path_prefix='/'):
         return list_groups(client=self.client, path_prefix=path_prefix)
 
@@ -93,6 +122,15 @@ class IamUtil(object):
     def list_roles(self, path_prefix='/'):
         return list_roles(client=self.client, path_prefix=path_prefix)
 
+    def list_users(self, path_prefix='/'):
+        return list_users(client=self.client, path_prefix=path_prefix)
+
+    def update_access_key(self, user_name, access_key_id, status):
+        return update_access_key(client=self.client, user_name=user_name, access_key_id=access_key_id, status=status)
+
+    def update_all_access_keys_for_user(self, user_name, status):
+        return update_all_access_keys_for_user(client=self.client, user_name=user_name, status=status)
+
     def update_account_password_policy(self, min_password_len=14, symbols=True, numbers=True, uppers=True,
                                        lowers=True, allow_change=True, max_age=90, previous_password_prevention=24,
                                        hard_expiry=False):
@@ -165,6 +203,32 @@ def attach_policy_to_role(client, role_name, policy_arn):
         raise IamUtilError(msg) from exc
 
 
+############################################################################
+# Methods for adding users to groups
+############################################################################
+
+
+def add_user_to_group(client, user_name, group_name):
+    """Add the user to the group
+
+    :param client: boto3.client object
+    :param user_name: (str) name of the role
+    :param group_name: (str) ARN of the policy
+    :return: None
+    :raises: IamUtilError
+    """
+    log = logging.getLogger(mod_logger + '.add_user_to_group')
+    log.info('Adding user [{u}] to group: [{g}]'.format(u=user_name, g=group_name))
+    try:
+        client.add_user_to_group(
+            GroupName=group_name,
+            UserName=user_name
+        )
+    except ClientError as exc:
+        msg = 'Problem adding user [{u}] to group [{g}]'.format(u=user_name, g=group_name)
+        raise IamUtilError(msg) from exc
+
+
 ############################################################################
 # Methods for creating groups
 ############################################################################
@@ -217,6 +281,58 @@ def create_group(client, group_name, path='/'):
     return response['Group']
 
 
+############################################################################
+# Methods for creating users
+############################################################################
+
+
+def create_user(client, user_name, path='/'):
+    """Creates the specified user, if it already exists returns it
+
+    :param client: boto3.client object
+    :param user_name: (str) user name
+    :param path: (str) path to the user
+    :return: (dict) group data (specified in boto3)
+    :raises: IamUtilError
+    """
+    log = logging.getLogger(mod_logger + '.create_user')
+    log.info('Attempting to create user [{n}]'.format(n=user_name))
+
+    # Checking for existing group
+    user_data = None
+    existing_users = list_users(client=client, path_prefix=path)
+    for existing_user in existing_users:
+        if 'UserName' not in existing_user.keys():
+            log.warning('UserName not found in user: {r}'.format(r=str(existing_user)))
+            continue
+        if 'Path' not in existing_user.keys():
+            log.warning('Path not found in user: {r}'.format(r=str(existing_user)))
+            continue
+        if 'Arn' not in existing_user.keys():
+            log.warning('Arn not found in user: {r}'.format(r=str(existing_user)))
+            continue
+        if user_name == existing_user['UserName'] and path == existing_user['Path']:
+            user_data = existing_user
+
+    if user_data:
+        log.info('Found existing user [{n}] at path [{p}]'.format(n=user_data, p=path))
+        return user_data
+    log.info('Attempting to create user [{n}]'.format(n=user_name))
+    try:
+        response = client.create_user(
+            UserName=user_name,
+            Path=path
+        )
+    except ClientError as exc:
+        msg = 'Problem creating user [{n}]'.format(n=user_name)
+        raise IamUtilError(msg) from exc
+    if 'User' not in response.keys():
+        msg = 'User not found in response: {d}'.format(d=str(response))
+        raise IamUtilError(msg)
+    log.info('Created new user: [{n}]'.format(n=user_name))
+    return response['User']
+
+
 ############################################################################
 # Methods for creating policies
 ############################################################################
@@ -551,6 +667,66 @@ def list_groups(client, path_prefix='/'):
     return group_list
 
 
+############################################################################
+# Methods for listing users
+############################################################################
+
+
+def list_users_with_marker(client, path_prefix='/', marker=None, max_results=100):
+    """Returns a list of IAM users using the provided marker
+
+    :param client: boto3.client object
+    :param path_prefix: (str) IAM user path prefix
+    :param max_results: (int) max results to query on
+    :param marker: (str) token to query on
+    :return: (dict) response object containing response data
+    """
+    if marker:
+        return client.list_users(
+            PathPrefix=path_prefix,
+            Marker=marker,
+            MaxItems=max_results
+        )
+    else:
+        return client.list_users(
+            PathPrefix=path_prefix,
+            MaxItems=max_results
+        )
+
+
+def list_users(client, path_prefix='/'):
+    """Lists users in IAM
+
+    :param client: boto3.client object
+    :param path_prefix: (str) IAM group path prefix
+    :return: (list) of groups (dict)
+    :raises: IamUtilError
+    """
+    log = logging.getLogger(mod_logger + '.list_users')
+    marker = None
+    next_query = True
+    user_list = []
+    log.info('Attempting to list IAM users...')
+    while True:
+        if not next_query:
+            break
+        response = list_users_with_marker(client=client, path_prefix=path_prefix, marker=marker)
+        if 'IsTruncated' not in response.keys():
+            log.warning('IsTruncated not found in response: {r}'.format(r=str(response)))
+            return user_list
+        if 'Users' not in response.keys():
+            log.warning('Users not found in response: {r}'.format(r=str(response)))
+            return user_list
+        next_query = response['IsTruncated']
+        user_list += response['Users']
+        if 'Marker' not in response.keys():
+            next_query = False
+        else:
+            marker = response['Marker']
+    log.info('Found {n} IAM users'.format(n=str(len(user_list))))
+    return user_list
+
+
 ############################################################################
 # Methods for listing policies
 ############################################################################
@@ -1039,6 +1215,190 @@ def add_policy_statement(client, policy_arn, statement):
     return True
 
 
+def add_source_ip_addresses_to_cons3rt_bucket_policy(client, policy_arn, source_ip_addresses):
+    """Adds the provided source IP addresses to the aws:SourceIp of the writeBuckets SID.  The statement will match
+    on the SID provided in the statement.
+    If the SID exists, the statement is not added.
+
+    :param client: boto3.client object
+    :param policy_arn: (str) policy ARN
+    :param source_ip_addresses: (list) source IP addresses
+    :return: (bool) True if successful or no update needed, False otherwise
+    :raises: IamUtilError
+    """
+    log = logging.getLogger(mod_logger + '.add_source_ip_addresses_to_cons3rt_bucket_policy')
+
+    # The writeBuckets Sid
+    write_buckets_sid = 'writeBuckets'
+
+    # Ensure the statement is a dict
+    if not isinstance(source_ip_addresses, list):
+        msg = 'source_ip_addresses expected to be type list, found: {t}'.format(t=str(type(source_ip_addresses)))
+        raise IamUtilError(msg)
+
+    # Validate the IP addresses
+    for source_ip_address in source_ip_addresses:
+        if not isinstance(source_ip_address, str):
+            msg = 'IP address in the source_ip_addresses must be a string, found: {t}'.format(
+                t=str(type(source_ip_address)))
+            raise IamUtilError(msg)
+        if not validate_ip_address(source_ip_address):
+            msg = 'Invalid IP address provided in the source_ip_addresses list arg: {i}'.format(i=source_ip_address)
+            raise IamUtilError(msg)
+
+    # Checking for existing policy
+    policy_data = get_policy_by_arn(client=client, policy_arn=policy_arn)
+    if 'PolicyName' not in policy_data.keys():
+        msg = 'PolicyName not found in policy data: {p}'.format(p=str(policy_data))
+        raise IamUtilError(msg)
+    policy_name = policy_data['PolicyName']
+
+    # Get the policy document
+    policy_document = get_default_policy_document(client=client, policy_arn=policy_arn)
+
+    # Ensure Statements exist
+    if 'Statement' not in policy_document.keys():
+        log.info('The default version policy document has no statements, nothing to do: {d}'.format(
+            d=str(policy_document)))
+        return True
+
+    # Get the list of statements
+    statements = policy_document['Statement']
+    log.info('Found {n} existing policy statements'.format(n=str(len(statements))))
+
+    # Get the writeBuckets statement
+    write_buckets_statement = None
+    for existing_statement in statements:
+        if 'Sid' not in existing_statement.keys():
+            log.warning('Sid not found in existing statement: {s}'.format(s=str(existing_statement)))
+            continue
+        if existing_statement['Sid'] == write_buckets_sid:
+            log.info('Found the write buckets SID: {s}'.format(s=write_buckets_sid))
+            write_buckets_statement = dict(existing_statement)
+
+    # Return if to writeBuckets SID was found
+    if not write_buckets_statement:
+        log.info('This policy has no {s} SID, nothing to do'.format(s=write_buckets_sid))
+        return True
+
+    # Ensure it has a condition
+    if 'Condition' not in write_buckets_statement.keys():
+        log.warning('Statement {s} found with no Condition: {d}'.format(
+            s=write_buckets_sid, d=str(write_buckets_statement)))
+        return False
+    condition = write_buckets_statement['Condition']
+
+    # Ensure IpAddress data exists
+    if 'IpAddress' not in condition.keys():
+        log.warning('Condition found with no IpAddress: {c}'.format(c=str(condition)))
+        return False
+    ip_address_data = condition['IpAddress']
+
+    # Ensure aws:SourceIp data exists
+    if 'aws:SourceIp' not in ip_address_data.keys():
+        log.warning('IpAddress found with no aws:SourceIp: {i}'.format(i=str(ip_address_data)))
+        return False
+
+    # Get the aws:SourceIp list
+    aws_source_ip_list = list(ip_address_data['aws:SourceIp'])
+
+    log.info('Adding policy source IPs [{i}] to the default version of policy ARN: {p}'.format(
+        p=policy_arn, i=','.join(source_ip_addresses)))
+
+    # Append IPs to the list
+    for source_ip_address in source_ip_addresses:
+        aws_source_ip_list.append(source_ip_address)
+
+    # Update the IpAddress data with the new list
+    ip_address_data['aws:SourceIp'] = list(aws_source_ip_list)
+
+    # update the policy with the new document
+    log.info('Updating policy [{n}] with new policy document: {d}'.format(n=policy_name, d=str(policy_document)))
+    try:
+        update_policy(client=client, policy_name=policy_name, policy_content=policy_document)
+    except IamUtilError as exc:
+        msg = 'Problem updating policy {n}'.format(n=policy_name)
+        raise IamUtilError(msg) from exc
+    return True
+
+
+def get_cons3rt_bucket_policy_source_ip_addresses(client, policy_arn):
+    """Returns a list of IP addresses in the writeBuckets statement (if it exists)
+
+    :param client: boto3.client object
+    :param policy_arn: (str) policy ARN
+    :return: (list) of source IPs in the writeBuckets statement
+    :raises: IamUtilError
+    """
+    log = logging.getLogger(mod_logger + '.get_cons3rt_bucket_policy_source_ip_addresses')
+
+    # Store the list of source IPs to return
+    bucket_source_ip_addresses = []
+
+    # The writeBuckets Sid
+    write_buckets_sid = 'writeBuckets'
+
+    # Checking for existing policy
+    policy_data = get_policy_by_arn(client=client, policy_arn=policy_arn)
+    if 'PolicyName' not in policy_data.keys():
+        msg = 'PolicyName not found in policy data: {p}'.format(p=str(policy_data))
+        raise IamUtilError(msg)
+    policy_name = policy_data['PolicyName']
+
+    # Get the policy document
+    policy_document = get_default_policy_document(client=client, policy_arn=policy_arn)
+
+    # Ensure Statements exist
+    if 'Statement' not in policy_document.keys():
+        log.info('The default version policy document has no statements, nothing to do: {d}'.format(
+            d=str(policy_document)))
+        return bucket_source_ip_addresses
+
+    # Get the list of statements
+    statements = policy_document['Statement']
+    log.info('Found {n} existing policy statements'.format(n=str(len(statements))))
+
+    # Get the writeBuckets statement
+    write_buckets_statement = None
+    for existing_statement in statements:
+        if 'Sid' not in existing_statement.keys():
+            log.warning('Sid not found in existing statement: {s}'.format(s=str(existing_statement)))
+            continue
+        if existing_statement['Sid'] == write_buckets_sid:
+            log.info('Found the write buckets SID: {s}'.format(s=write_buckets_sid))
+            write_buckets_statement = dict(existing_statement)
+
+    # Return if to writeBuckets SID was found
+    if not write_buckets_statement:
+        log.info('This policy has no {s} SID, nothing to do'.format(s=write_buckets_sid))
+        return bucket_source_ip_addresses
+
+    # Ensure it has a condition
+    if 'Condition' not in write_buckets_statement.keys():
+        log.warning('Statement {s} found with no Condition: {d}'.format(
+            s=write_buckets_sid, d=str(write_buckets_statement)))
+        return bucket_source_ip_addresses
+    condition = write_buckets_statement['Condition']
+
+    # Ensure IpAddress data exists
+    if 'IpAddress' not in condition.keys():
+        log.warning('Condition found with no IpAddress: {c}'.format(c=str(condition)))
+        return bucket_source_ip_addresses
+    ip_address_data = condition['IpAddress']
+
+    # Ensure aws:SourceIp data exists
+    if 'aws:SourceIp' not in ip_address_data.keys():
+        log.warning('IpAddress found with no aws:SourceIp: {i}'.format(i=str(ip_address_data)))
+        return bucket_source_ip_addresses
+
+    # Get the aws:SourceIp list
+    bucket_source_ip_addresses = list(ip_address_data['aws:SourceIp'])
+    log.info('Found IP address source list in policy [{n}]: {i}'.format(
+        n=policy_name, i=','.join(bucket_source_ip_addresses)
+    ))
+    return bucket_source_ip_addresses
+
+
 ############################################################################
 # Update an existing role
 ############################################################################
@@ -1119,3 +1479,191 @@ def update_role_trust_policy(client, role_name, role_policy):
         msg = 'Problem updating the trust policy to [{p}] for role [{n}] with\n{d}'.format(
             p=role_policy, n=role_name, d=str(json_role_policy_data))
         raise IamUtilError(msg) from exc
+
+
+############################################################################
+# IAM access keys
+############################################################################
+
+def create_access_key_for_user(client, user_name):
+    """Creates an access key for the specified user
+
+    :param client: boto3.client object
+    :param user_name: (str) user name
+    :return: (dict) Access Key data
+    :raises: IamUtilError
+    """
+    log = logging.getLogger(mod_logger + '.create_access_key_for_user')
+    log.info('Attempting to create access key for user: {n}'.format(n=user_name))
+    try:
+        response = client.create_access_key(UserName=user_name)
+    except ClientError as exc:
+        msg = 'Problem creating access key for user [{n}]'.format(n=user_name)
+        raise IamUtilError(msg) from exc
+    if 'AccessKey' not in response.keys():
+        msg = 'AccessKey not found in response: {d}'.format(d=str(response))
+        raise IamUtilError(msg)
+    access_key = response['AccessKey']
+    if 'AccessKeyId' not in access_key:
+        msg = 'AccessKeyId not found in access key data: {d}'.format(d=str(access_key))
+        raise IamUtilError(msg)
+    access_key_id = response['AccessKey']['AccessKeyId']
+    log.info('Created new access key for user [{n}] with ID: {i}'.format(n=user_name, i=access_key_id))
+    return response['AccessKey']
+
+
+def create_first_access_key_for_user(client, user_name):
+    """Checks for existing access keys, and only creates one if none are found
+
+    :param client: boto3.client object
+    :param user_name: user name
+    :return: (list) Access Key data, either existing ones or the new one
+    :raises: IamUtilError
+    """
+    log = logging.getLogger(mod_logger + '.create_first_access_key_for_user')
+    log.info('Creating an access key for user [{n}] only if one does not exist yet'.format(n=user_name))
+    existing_access_keys = list_access_keys_for_user(client=client, user_name=user_name)
+    if len(existing_access_keys) > 0:
+        log.info('Found existing access key IDs for user [{n}]:'.format(n=user_name))
+        for existing_access_key in existing_access_keys:
+            log.info('Access Key ID: {i}'.format(i=existing_access_key['AccessKeyId']))
+        return existing_access_keys
+    else:
+        log.info('No existing access keys found for user [{u}], creating one...'.format(u=user_name))
+        return [create_access_key_for_user(client=client, user_name=user_name)]
+
+
+def delete_access_key(client, user_name, access_key_id):
+    """Delete an access key for the specified user
+
+    :param client: boto3.client object
+    :param user_name: (str) user name
+    :param access_key_id: (str) ID of the access key
+    :return: (dict) Access Key data
+    :raises: IamUtilError
+    """
+    log = logging.getLogger(mod_logger + '.delete_access_key')
+    log.info('Deleting access key ID [{i}] for user: {n}'.format(i=access_key_id, n=user_name))
+    try:
+        client.delete_access_key(UserName=user_name, AccessKeyId=access_key_id)
+    except ClientError as exc:
+        msg = 'Problem deleting access key ID [{i}] for user [{n}]'.format(i=access_key_id, n=user_name)
+        raise IamUtilError(msg) from exc
+
+
+def delete_all_access_keys_for_user(client, user_name):
+    """Delete an access key for the specified user
+
+    :param client: boto3.client object
+    :param user_name: (str) user name
+    :return: (list) Deleted access Key data
+    :raises: IamUtilError
+    """
+    log = logging.getLogger(mod_logger + '.delete_all_access_keys_for_user')
+    log.info('Deleting all access key IDs for user: {n}'.format(n=user_name))
+    existing_access_keys = list_access_keys_for_user(client=client, user_name=user_name)
+    log.info('Found {n} existing access key IDs for user [{u}]:'.format(n=str(len(existing_access_keys)), u=user_name))
+    for existing_access_key in existing_access_keys:
+        delete_access_key(client=client, user_name=user_name, access_key_id=existing_access_key['AccessKeyId'])
+    return existing_access_keys
+
+
+def list_access_keys_for_user_with_marker(client, user_name, marker=None, max_results=100):
+    """Returns a list of access keys for the provided user using the provided marker
+
+    :param client: boto3.client object
+    :param user_name: (str) user name
+    :param max_results: (int) max results to query on
+    :param marker: (str) token to query on
+    :return: (dict) response object containing response data
+    """
+    if marker:
+        return client.list_access_keys(
+            UserName=user_name,
+            Marker=marker,
+            MaxItems=max_results
+        )
+    else:
+        return client.list_access_keys(
+            UserName=user_name,
+            MaxItems=max_results
+        )
+
+
+def list_access_keys_for_user(client, user_name):
+    """Lists access keys for user
+
+    :param client: boto3.client object
+    :param user_name: (str) user name
+    :return: (list) of groups (dict)
+    :raises: IamUtilError
+    """
+    log = logging.getLogger(mod_logger + '.list_access_keys_for_user')
+    marker = None
+    next_query = True
+    access_key_list = []
+    log.info('Attempting to list access keys for user: {n}'.format(n=user_name))
+    while True:
+        if not next_query:
+            break
+        response = list_access_keys_for_user_with_marker(client=client, user_name=user_name, marker=marker)
+        if 'IsTruncated' not in response.keys():
+            log.warning('IsTruncated not found in response: {r}'.format(r=str(response)))
+            return access_key_list
+        if 'AccessKeyMetadata' not in response.keys():
+            log.warning('AccessKeyMetadata not found in response: {r}'.format(r=str(response)))
+            return access_key_list
+        next_query = response['IsTruncated']
+        access_key_list += response['AccessKeyMetadata']
+        if 'Marker' not in response.keys():
+            next_query = False
+        else:
+            marker = response['Marker']
+    log.info('Found {n} IAM access keys for user: {u}'.format(u=user_name, n=str(len(access_key_list))))
+    return access_key_list
+
+
+def update_access_key(client, user_name, access_key_id, status):
+    """Delete an access key for the specified user
+
+    :param client: boto3.client object
+    :param user_name: (str) user name
+    :param access_key_id: (str) ID of the access key
+    :param status: (str) Active or Inactive
+    :return: (dict) Access Key data
+    :raises: IamUtilError
+    """
+    log = logging.getLogger(mod_logger + '.update_access_key')
+
+    # Ensure status is valid
+    if status not in ['Active', 'Inactive']:
+        msg = 'Invalid status provided [{s}], must be Active or Inactive'.format(s=status)
+        raise IamUtilError(msg)
+
+    # Update access key status
+    log.info('Updating access key ID [{i}] to [{s}] for user: {n}'.format(i=access_key_id, s=status, n=user_name))
+    try:
+        client.update_access_key(UserName=user_name, AccessKeyId=access_key_id, Status=status)
+    except ClientError as exc:
+        msg = 'Problem updating status of access key ID [{i}] for user [{n}] to: {s}'.format(
+            i=access_key_id, n=user_name, s=status)
+        raise IamUtilError(msg) from exc
+
+
+def update_all_access_keys_for_user(client, user_name, status):
+    """Delete an access key for the specified user
+
+    :param client: boto3.client object
+    :param user_name: (str) user name
+    :param status: (str) status of the access key
+    :return: (list) Deleted access Key data
+    :raises: IamUtilError
+    """
+    log = logging.getLogger(mod_logger + '.update_all_access_keys_for_user')
+    log.info('Updating all access key IDs for user [{n}] with status: {s}'.format(n=user_name, s=status))
+    existing_access_keys = list_access_keys_for_user(client=client, user_name=user_name)
+    log.info('Found {n} existing access key IDs for user [{u}]:'.format(n=str(len(existing_access_keys)), u=user_name))
+    for existing_access_key in existing_access_keys:
+        update_access_key(client=client, user_name=user_name, access_key_id=existing_access_key['AccessKeyId'],
+                          status=status)
+    return existing_access_keys
